@@ -1,6 +1,6 @@
 /******************************************************************************
  *
- * Module Name: aslload - compiler namespace load callbacks
+ * Module Name: dswload - Dispatcher namespace load callbacks
  *
  *****************************************************************************/
 
@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2020, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2018, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -164,7 +164,6 @@
 
 static ACPI_STATUS
 LdLoadFieldElements (
-    UINT32                  AmlType,
     ACPI_PARSE_OBJECT       *Op,
     ACPI_WALK_STATE         *WalkState);
 
@@ -191,18 +190,6 @@ LdCommonNamespaceEnd (
     UINT32                  Level,
     void                    *Context);
 
-static void
-LdCheckSpecialNames (
-    ACPI_NAMESPACE_NODE     *Node,
-    ACPI_PARSE_OBJECT       *Op);
-
-static ACPI_STATUS
-LdAnalyzeExternals (
-    ACPI_NAMESPACE_NODE     *Node,
-    ACPI_PARSE_OBJECT       *Op,
-    ACPI_OBJECT_TYPE        ExternalOpType,
-    ACPI_WALK_STATE         *WalkState);
-
 
 /*******************************************************************************
  *
@@ -246,11 +233,7 @@ LdLoadNamespace (
 
     /* Dump the namespace if debug is enabled */
 
-    if (AcpiDbgLevel & ACPI_LV_TABLES)
-    {
-        AcpiNsDumpTables (ACPI_NS_ALL, ACPI_UINT32_MAX);
-    }
-
+    AcpiNsDumpTables (ACPI_NS_ALL, ACPI_UINT32_MAX);
     ACPI_FREE (WalkState);
     return (AE_OK);
 }
@@ -260,8 +243,7 @@ LdLoadNamespace (
  *
  * FUNCTION:    LdLoadFieldElements
  *
- * PARAMETERS:  AmlType         - Type to search
- *              Op              - Parent node (Field)
+ * PARAMETERS:  Op              - Parent node (Field)
  *              WalkState       - Current walk state
  *
  * RETURN:      Status
@@ -273,33 +255,14 @@ LdLoadNamespace (
 
 static ACPI_STATUS
 LdLoadFieldElements (
-    UINT32                  AmlType,
     ACPI_PARSE_OBJECT       *Op,
     ACPI_WALK_STATE         *WalkState)
 {
     ACPI_PARSE_OBJECT       *Child = NULL;
-    ACPI_PARSE_OBJECT       *SourceRegion;
     ACPI_NAMESPACE_NODE     *Node;
     ACPI_STATUS             Status;
-    char                    *ExternalPath;
 
 
-    SourceRegion = UtGetArg (Op, 0);
-    if (SourceRegion)
-    {
-        Status = AcpiNsLookup (WalkState->ScopeInfo,
-            SourceRegion->Asl.Value.String, AmlType, ACPI_IMODE_EXECUTE,
-            ACPI_NS_SEARCH_PARENT | ACPI_NS_DONT_OPEN_SCOPE, NULL, &Node);
-        if (Status == AE_NOT_FOUND)
-        {
-            /*
-             * If the named object is not found, it means that it is either a
-             * forward reference or the named object does not exist.
-             */
-            SourceRegion->Asl.CompileFlags |= OP_NOT_FOUND_DURING_LOAD;
-        }
-    }
-
     /* Get the first named field element */
 
     switch (Op->Asl.AmlOpcode)
@@ -354,10 +317,10 @@ LdLoadFieldElements (
                     return (Status);
                 }
                 else if (Status == AE_ALREADY_EXISTS &&
-                    (Node->Flags & ANOBJ_IS_EXTERNAL))
+                    (Node->Flags & ANOBJ_IS_EXTERNAL) &&
+                    Node->OwnerId != WalkState->OwnerId)
                 {
                     Node->Type = (UINT8) ACPI_TYPE_LOCAL_REGION_FIELD;
-                    Node->Flags &= ~ANOBJ_IS_EXTERNAL;
                 }
                 else
                 {
@@ -365,16 +328,9 @@ LdLoadFieldElements (
                      * The name already exists in this scope
                      * But continue processing the elements
                      */
-                    ExternalPath = AcpiNsGetNormalizedPathname (Node, TRUE);
-
-                    AslDualParseOpError (ASL_ERROR, ASL_MSG_NAME_EXISTS, Child,
-                        ExternalPath, ASL_MSG_FOUND_HERE, Node->Op,
-                        ExternalPath);
-
-                    if (ExternalPath)
-                    {
-                        ACPI_FREE (ExternalPath);
-                    }
+                    AslDualParseOpError (ASL_WARNING, ASL_MSG_EXTERN_COLLISION, Child,
+                        Child->Asl.Value.String, ASL_MSG_EXTERN_FOUND_HERE, Node->Op,
+                        Node->Op->Asl.ExternalName);
                 }
             }
             else
@@ -418,7 +374,6 @@ LdLoadResourceElements (
     ACPI_PARSE_OBJECT       *InitializerOp = NULL;
     ACPI_NAMESPACE_NODE     *Node;
     ACPI_STATUS             Status;
-    char                    *ExternalPath;
 
 
     /*
@@ -435,17 +390,10 @@ LdLoadResourceElements (
         {
             /* Actual node causing the error was saved in ParentMethod */
 
-            ExternalPath = AcpiNsGetNormalizedPathname (Node, TRUE);
-
             AslDualParseOpError (ASL_ERROR, ASL_MSG_NAME_EXISTS,
                 (ACPI_PARSE_OBJECT *) Op->Asl.ParentMethod,
-                ExternalPath, ASL_MSG_FOUND_HERE, Node->Op,
-                ExternalPath);
-
-            if (ExternalPath)
-            {
-                ACPI_FREE (ExternalPath);
-            }
+                Op->Asl.Namepath, ASL_MSG_FOUND_HERE, Node->Op,
+                Node->Op->Asl.ExternalName);
             return (AE_OK);
         }
         return (Status);
@@ -466,8 +414,10 @@ LdLoadResourceElements (
         {
             Status = AcpiNsLookup (WalkState->ScopeInfo,
                 InitializerOp->Asl.ExternalName,
-                ACPI_TYPE_LOCAL_RESOURCE_FIELD, ACPI_IMODE_LOAD_PASS1,
-                ACPI_NS_NO_UPSEARCH | ACPI_NS_DONT_OPEN_SCOPE, NULL, &Node);
+                ACPI_TYPE_LOCAL_RESOURCE_FIELD,
+                ACPI_IMODE_LOAD_PASS1,
+                ACPI_NS_NO_UPSEARCH | ACPI_NS_DONT_OPEN_SCOPE,
+                NULL, &Node);
             if (ACPI_FAILURE (Status))
             {
                 return (Status);
@@ -514,14 +464,15 @@ LdNamespace1Begin (
     ACPI_PARSE_OBJECT       *MethodOp;
     ACPI_STATUS             Status;
     ACPI_OBJECT_TYPE        ObjectType;
+    ACPI_OBJECT_TYPE        ActualObjectType = ACPI_TYPE_ANY;
     char                    *Path;
     UINT32                  Flags = ACPI_NS_NO_UPSEARCH;
     ACPI_PARSE_OBJECT       *Arg;
     UINT32                  i;
     BOOLEAN                 ForceNewScope = FALSE;
+    ACPI_OWNER_ID           OwnerId = 0;
     const ACPI_OPCODE_INFO  *OpInfo;
     ACPI_PARSE_OBJECT       *ParentOp;
-    char                    *ExternalPath;
 
 
     ACPI_FUNCTION_NAME (LdNamespace1Begin);
@@ -530,31 +481,45 @@ LdNamespace1Begin (
     ACPI_DEBUG_PRINT ((ACPI_DB_DISPATCH, "Op %p [%s]\n",
         Op, Op->Asl.ParseOpName));
 
+    if (Op->Asl.ParseOpcode == PARSEOP_DEFINITION_BLOCK)
+    {
+        /*
+         * Allocate an OwnerId for this block. This helps identify the owners
+         * of each namespace node. This is used in determining whether if
+         * certain external declarations cause redefinition errors.
+         */
+        Status = AcpiUtAllocateOwnerId (&OwnerId);
+        WalkState->OwnerId = OwnerId;
+        if (ACPI_FAILURE (Status))
+        {
+            AslCoreSubsystemError (Op, Status,
+                "Failure to allocate owner ID to this definition block.", FALSE);
+            return_ACPI_STATUS (Status);
+        }
+    }
+
     /*
      * We are only interested in opcodes that have an associated name
      * (or multiple names)
      */
     switch (Op->Asl.AmlOpcode)
     {
-    case AML_INDEX_FIELD_OP:
-
-        Status = LdLoadFieldElements (ACPI_TYPE_LOCAL_REGION_FIELD, Op, WalkState);
-        return (Status);
-
     case AML_BANK_FIELD_OP:
+    case AML_INDEX_FIELD_OP:
     case AML_FIELD_OP:
 
-        Status = LdLoadFieldElements (ACPI_TYPE_REGION, Op, WalkState);
+        Status = LdLoadFieldElements (Op, WalkState);
         return (Status);
 
     case AML_INT_CONNECTION_OP:
 
+
         if (Op->Asl.Child->Asl.AmlOpcode != AML_INT_NAMEPATH_OP)
         {
             break;
         }
-
         Arg = Op->Asl.Child;
+
         Status = AcpiNsLookup (WalkState->ScopeInfo, Arg->Asl.ExternalName,
             ACPI_TYPE_ANY, ACPI_IMODE_EXECUTE, ACPI_NS_SEARCH_PARENT,
             WalkState, &Node);
@@ -563,6 +528,15 @@ LdNamespace1Begin (
             break;
         }
 
+        if (Node->Type == ACPI_TYPE_BUFFER)
+        {
+            Arg->Asl.Node = Node;
+
+            Arg = Node->Op->Asl.Child;  /* Get namepath */
+            Arg = Arg->Asl.Next;        /* Get actual buffer */
+            Arg = Arg->Asl.Child;       /* Buffer length */
+            Arg = Arg->Asl.Next;        /* RAW_DATA buffer */
+        }
         break;
 
     default:
@@ -582,14 +556,14 @@ LdNamespace1Begin (
     /* Check for a possible illegal forward reference */
 
     if ((Op->Asl.ParseOpcode == PARSEOP_NAMESEG) ||
-        (Op->Asl.ParseOpcode == PARSEOP_NAMESTRING) ||
-        (Op->Asl.ParseOpcode == PARSEOP_METHODCALL))
+        (Op->Asl.ParseOpcode == PARSEOP_NAMESTRING))
     {
         /*
          * Op->Asl.Namepath will be NULL for these opcodes.
          * These opcodes are guaranteed to have a parent.
          * Examine the parent opcode.
          */
+        Status = AE_OK;
         ParentOp = Op->Asl.Parent;
         OpInfo = AcpiPsGetOpcodeInfo (ParentOp->Asl.AmlOpcode);
 
@@ -599,8 +573,7 @@ LdNamespace1Begin (
          * We only want references to named objects:
          *      Store (2, WXYZ) -> Attempt to resolve the name
          */
-        if ((Op->Asl.ParseOpcode != PARSEOP_METHODCALL) &&
-            (OpInfo->Class == AML_CLASS_NAMED_OBJECT))
+        if (OpInfo->Class == AML_CLASS_NAMED_OBJECT)
         {
             return (AE_OK);
         }
@@ -619,7 +592,7 @@ LdNamespace1Begin (
         if (Status == AE_NOT_FOUND)
         {
             /*
-             * This is either a forward reference or the object truly
+             * This is either a foward reference or the object truly
              * does not exist. The two cases can only be differentiated
              * during the cross-reference stage later. Mark the Op/Name
              * as not-found for now to indicate the need for further
@@ -687,7 +660,8 @@ LdNamespace1Begin (
          *
          * first child is name, next child is ObjectType
          */
-        ObjectType = (UINT8) Op->Asl.Child->Asl.Next->Asl.Value.Integer;
+        ActualObjectType = (UINT8) Op->Asl.Child->Asl.Next->Asl.Value.Integer;
+        ObjectType = ACPI_TYPE_ANY;
 
         /*
          * We will mark every new node along the path as "External". This
@@ -706,7 +680,7 @@ LdNamespace1Begin (
          *       Store (\_SB_.PCI0.ABCD, Local0)
          *   }
          */
-        Flags |= ACPI_NS_EXTERNAL | ACPI_NS_DONT_OPEN_SCOPE;
+        Flags |= ACPI_NS_EXTERNAL;
         break;
 
     case PARSEOP_DEFAULT_ARG:
@@ -728,7 +702,8 @@ LdNamespace1Begin (
          * handle this case. Perhaps someday this case can go away.
          */
         Status = AcpiNsLookup (WalkState->ScopeInfo, Path, ACPI_TYPE_ANY,
-            ACPI_IMODE_EXECUTE, ACPI_NS_SEARCH_PARENT, WalkState, &Node);
+            ACPI_IMODE_EXECUTE, ACPI_NS_SEARCH_PARENT,
+            WalkState, &(Node));
         if (ACPI_FAILURE (Status))
         {
             if (Status == AE_NOT_FOUND)
@@ -736,26 +711,23 @@ LdNamespace1Begin (
                 /* The name was not found, go ahead and create it */
 
                 Status = AcpiNsLookup (WalkState->ScopeInfo, Path,
-                    ACPI_TYPE_LOCAL_SCOPE, ACPI_IMODE_LOAD_PASS1,
-                    Flags, WalkState, &Node);
+                    ACPI_TYPE_LOCAL_SCOPE,
+                    ACPI_IMODE_LOAD_PASS1, Flags,
+                    WalkState, &(Node));
                 if (ACPI_FAILURE (Status))
                 {
                     return_ACPI_STATUS (Status);
                 }
 
-                /* However, this is an error -- operand to Scope must exist */
-
-                if (strlen (Op->Asl.ExternalName) == ACPI_NAMESEG_SIZE)
-                {
-                    AslError (ASL_ERROR, ASL_MSG_NOT_FOUND, Op,
-                        Op->Asl.ExternalName);
-                }
-                else
-                {
-                    AslError (ASL_ERROR, ASL_MSG_NAMEPATH_NOT_EXIST, Op,
-                        Op->Asl.ExternalName);
-                }
-
+                /*
+                 * However, this is an error -- primarily because the MS
+                 * interpreter can't handle a forward reference from the
+                 * Scope() operator.
+                 */
+                AslError (ASL_ERROR, ASL_MSG_NOT_FOUND, Op,
+                    Op->Asl.ExternalName);
+                AslError (ASL_ERROR, ASL_MSG_SCOPE_FWD_REF, Op,
+                    Op->Asl.ExternalName);
                 goto FinishNode;
             }
 
@@ -774,7 +746,7 @@ LdNamespace1Begin (
              * 10/2015.
              */
             if ((Node->Flags & ANOBJ_IS_EXTERNAL) &&
-                (ACPI_COMPARE_NAMESEG (AslGbl_TableSignature, "DSDT")))
+                (ACPI_COMPARE_NAME (Gbl_TableSignature, "DSDT")))
             {
                 /* However, allowed if the reference is within a method */
 
@@ -820,9 +792,9 @@ LdNamespace1Begin (
              * Which is used to workaround the fact that the MS interpreter
              * does not allow Scope() forward references.
              */
-            sprintf (AslGbl_MsgBuffer, "%s [%s], changing type to [Scope]",
+            sprintf (MsgBuffer, "%s [%s], changing type to [Scope]",
                 Op->Asl.ExternalName, AcpiUtGetTypeName (Node->Type));
-            AslError (ASL_REMARK, ASL_MSG_SCOPE_TYPE, Op, AslGbl_MsgBuffer);
+            AslError (ASL_REMARK, ASL_MSG_SCOPE_TYPE, Op, MsgBuffer);
 
             /* Switch the type to scope, open the new scope */
 
@@ -839,9 +811,9 @@ LdNamespace1Begin (
 
             /* All other types are an error */
 
-            sprintf (AslGbl_MsgBuffer, "%s [%s]", Op->Asl.ExternalName,
+            sprintf (MsgBuffer, "%s [%s]", Op->Asl.ExternalName,
                 AcpiUtGetTypeName (Node->Type));
-            AslError (ASL_ERROR, ASL_MSG_SCOPE_TYPE, Op, AslGbl_MsgBuffer);
+            AslError (ASL_ERROR, ASL_MSG_SCOPE_TYPE, Op, MsgBuffer);
 
             /*
              * However, switch the type to be an actual scope so
@@ -867,6 +839,7 @@ LdNamespace1Begin (
         break;
     }
 
+
     ACPI_DEBUG_PRINT ((ACPI_DB_DISPATCH, "Loading name: %s, (%s)\n",
         Op->Asl.ExternalName, AcpiUtGetTypeName (ObjectType)));
 
@@ -874,18 +847,6 @@ LdNamespace1Begin (
 
     Flags |= ACPI_NS_ERROR_IF_FOUND;
 
-    /*
-     * For opcodes that enter new names into the namespace,
-     * all prefix NameSegs must exist.
-     */
-    WalkState->OpInfo = AcpiPsGetOpcodeInfo (Op->Asl.AmlOpcode);
-    if (((WalkState->OpInfo->Flags & AML_NAMED) ||
-        (WalkState->OpInfo->Flags & AML_CREATE)) &&
-        (Op->Asl.AmlOpcode != AML_EXTERNAL_OP))
-    {
-        Flags |= ACPI_NS_PREFIX_MUST_EXIST;
-    }
-
     /*
      * Enter the named type into the internal namespace. We enter the name
      * as we go downward in the parse tree. Any necessary subobjects that
@@ -907,79 +868,128 @@ LdNamespace1Begin (
                 Node->Type = (UINT8) ObjectType;
                 Status = AE_OK;
             }
-            else if ((Node->Flags & ANOBJ_IS_EXTERNAL) ||
-                     (Op->Asl.ParseOpcode == PARSEOP_EXTERNAL))
+            else if ((Node->Flags & ANOBJ_IS_EXTERNAL) &&
+                     (Op->Asl.ParseOpcode != PARSEOP_EXTERNAL))
             {
-                Status = LdAnalyzeExternals (Node, Op, ObjectType, WalkState);
-                if (ACPI_FAILURE (Status))
+                /*
+                 * Allow one create on an object or segment that was
+                 * previously declared External only if WalkState->OwnerId and
+                 * Node->OwnerId are different (meaning that the current WalkState
+                 * and the Node are in different tables).
+                 */
+                Node->Flags &= ~ANOBJ_IS_EXTERNAL;
+                Node->Type = (UINT8) ObjectType;
+
+                /* Just retyped a node, probably will need to open a scope */
+
+                if (AcpiNsOpensScope (ObjectType))
                 {
-                    if (Status == AE_ERROR)
+                    Status = AcpiDsScopeStackPush (Node, ObjectType, WalkState);
+                    if (ACPI_FAILURE (Status))
                     {
-                        /*
-                         * The use of AE_ERROR here indicates that there was a
-                         * compiler error emitted in LdAnalyzeExternals which
-                         * means that the caller should proceed to the next Op
-                         * for analysis of subsequent parse objects.
-                         */
-                        Status = AE_OK;
+                        return_ACPI_STATUS (Status);
                     }
-                    return_ACPI_STATUS (Status);
                 }
 
-                if (!(Node->Flags & ANOBJ_IS_EXTERNAL) &&
-                     (Op->Asl.ParseOpcode == PARSEOP_EXTERNAL))
+                Status = AE_OK;
+
+                if (Node->OwnerId == WalkState->OwnerId &&
+                    !(Node->Flags & IMPLICIT_EXTERNAL))
                 {
-                    /*
-                     * If we get to here, it means that an actual definition of
-                     * the object declared external exists. Meaning that Op
-                     * loading this this Op should have no change to the ACPI
-                     * namespace. By going to FinishNode, we skip the
-                     * assignment of Node->Op = Op.
-                     */
-                    goto FinishNode;
+                    AslDualParseOpError (ASL_WARNING, ASL_MSG_EXTERN_COLLISION, Op,
+                        Op->Asl.ExternalName, ASL_MSG_EXTERN_FOUND_HERE, Node->Op,
+                        Node->Op->Asl.ExternalName);
+                }
+                if (Node->Flags & IMPLICIT_EXTERNAL)
+                {
+                    Node->Flags &= ~IMPLICIT_EXTERNAL;
+                }
+            }
+            else if (!(Node->Flags & ANOBJ_IS_EXTERNAL) &&
+                     (Op->Asl.ParseOpcode == PARSEOP_EXTERNAL))
+            {
+                /*
+                 * Allow externals in same scope as the definition of the
+                 * actual object. Similar to C. Allows multiple definition
+                 * blocks that refer to each other in the same file. However,
+                 * do not allow name declaration and an external declaration
+                 * within the same table. This is considered a re-declaration.
+                 */
+                Status = AE_OK;
+
+                if (Node->OwnerId == WalkState->OwnerId)
+                {
+                    AslDualParseOpError (ASL_WARNING, ASL_MSG_EXTERN_COLLISION, Op,
+                        Op->Asl.ExternalName, ASL_MSG_EXTERN_FOUND_HERE, Node->Op,
+                        Node->Op->Asl.ExternalName);
+                }
+            }
+            else if ((Node->Flags & ANOBJ_IS_EXTERNAL) &&
+                     (Op->Asl.ParseOpcode == PARSEOP_EXTERNAL) &&
+                     (ObjectType == ACPI_TYPE_ANY))
+            {
+                /*
+                 * Allow update of externals of unknown type.
+                 * In the case that multiple definition blocks are being
+                 * parsed, updating the OwnerId allows enables subsequent calls
+                 * of this method to understand which table the most recent
+                 * external declaration was seen. Without this OwnerId update,
+                 * code like the following is allowed to compile:
+                 *
+                 * DefinitionBlock("externtest.aml", "DSDT", 0x02, "Intel", "Many", 0x00000001)
+                 * {
+                 *     External(ERRS,methodobj)
+                 *     Method (MAIN)
+                 *     {
+                 *         Name(NUM2, 0)
+                 *         ERRS(1,2,3)
+                 *     }
+                 * }
+                 *
+                 * DefinitionBlock("externtest.aml", "SSDT", 0x02, "Intel", "Many", 0x00000001)
+                 * {
+                 *     if (0)
+                 *     {
+                 *         External(ERRS,methodobj)
+                 *     }
+                 *     Method (ERRS,3)
+                 *     {}
+                 *
+                 * }
+                 */
+                Node->OwnerId = WalkState->OwnerId;
+
+                if (AcpiNsOpensScope (ActualObjectType))
+                {
+                    Node->Type = (UINT8) ActualObjectType;
+                    Status = AE_OK;
+                }
+                else
+                {
+                    sprintf (MsgBuffer, "%s [%s]", Op->Asl.ExternalName,
+                        AcpiUtGetTypeName (Node->Type));
+                    AslError (ASL_ERROR, ASL_MSG_SCOPE_TYPE, Op, MsgBuffer);
+                    return_ACPI_STATUS (AE_OK);
                 }
             }
             else
             {
                 /* Valid error, object already exists */
 
-                ExternalPath = AcpiNsGetNormalizedPathname (Node, TRUE);
-
                 AslDualParseOpError (ASL_ERROR, ASL_MSG_NAME_EXISTS, Op,
-                    ExternalPath, ASL_MSG_FOUND_HERE, Node->Op,
-                    ExternalPath);
-
-                if (ExternalPath)
-                {
-                    ACPI_FREE (ExternalPath);
-                }
+                    Op->Asl.ExternalName, ASL_MSG_FOUND_HERE, Node->Op,
+                    Node->Op->Asl.ExternalName);
                 return_ACPI_STATUS (AE_OK);
             }
         }
-        else if (AE_NOT_FOUND)
-        {
-            /*
-             * One or more prefix NameSegs of the NamePath do not exist
-             * (all of them must exist). Attempt to continue compilation
-             * by setting the current scope to the root.
-             */
-            Node = AcpiGbl_RootNode;
-            Status = AE_OK;
-        }
         else
         {
-            /* Flag all other errors as coming from the ACPICA core */
-
             AslCoreSubsystemError (Op, Status,
                 "Failure from namespace lookup", FALSE);
             return_ACPI_STATUS (Status);
         }
     }
 
-    /* Check special names like _WAK and _PTS */
-
-    LdCheckSpecialNames (Node, Op);
-
     if (ForceNewScope)
     {
         Status = AcpiDsScopeStackPush (Node, ObjectType, WalkState);
@@ -989,15 +999,21 @@ LdNamespace1Begin (
         }
     }
 
-    /* Point the Node back to the original Parse node */
-
+FinishNode:
+    /*
+     * Point the parse node to the new namespace node, and point
+     * the Node back to the original Parse node
+     */
+    Op->Asl.Node = Node;
     Node->Op = Op;
 
-FinishNode:
+    /* Set the actual data type if appropriate (EXTERNAL term only) */
 
-    /* Point the parse node to the new namespace node */
-
-    Op->Asl.Node = Node;
+    if (ActualObjectType != ACPI_TYPE_ANY)
+    {
+        Node->Type = (UINT8) ActualObjectType;
+        Node->Value = ASL_EXTERNAL_METHOD;
+    }
 
     if (Op->Asl.ParseOpcode == PARSEOP_METHOD)
     {
@@ -1007,259 +1023,11 @@ FinishNode:
          */
         Node->Value = (UINT32) Op->Asl.Extra;
     }
-    else if (Op->Asl.ParseOpcode == PARSEOP_EXTERNAL &&
-        Node->Type == ACPI_TYPE_METHOD &&
-        (Node->Flags & ANOBJ_IS_EXTERNAL))
-    {
-        Node->Value =
-            (UINT32) Op->Asl.Child->Asl.Next->Asl.Next->Asl.Value.Integer;
-    }
 
     return_ACPI_STATUS (Status);
 }
 
 
-/*******************************************************************************
- *
- * FUNCTION:    LdMatchExternType
- *
- * PARAMETERS:  Type1
- *              Type2
- *
- * RETURN:      BOOLEAN
- *
- * DESCRIPTION: Match Type1 and Type2 with the assumption that one might be
- *              using external types and another might be using local types.
- *              This should be used to compare the types found in external
- *              declarations with types found in other external declarations or
- *              named object declaration. This should not be used to match two
- *              object type declarations.
- *
- ******************************************************************************/
-
-static BOOLEAN
-LdMatchExternType (
-    ACPI_OBJECT_TYPE        Type1,
-    ACPI_OBJECT_TYPE        Type2)
-{
-    BOOLEAN                 Type1IsLocal = Type1 > ACPI_TYPE_EXTERNAL_MAX;
-    BOOLEAN                 Type2IsLocal = Type2 > ACPI_TYPE_EXTERNAL_MAX;
-    ACPI_OBJECT_TYPE        ExternalType;
-    ACPI_OBJECT_TYPE        LocalType;
-
-
-    /*
-     * The inputs could represent types that are local to ACPICA or types that
-     * are known externally. Some local types, such as the OperationRegion
-     * field units, are defined with more granularity than ACPICA local types.
-     *
-     * Therefore, map the local types to the external types before matching.
-     */
-    if (Type1IsLocal && !Type2IsLocal)
-    {
-        LocalType = Type1;
-        ExternalType = Type2;
-    }
-    else if (!Type1IsLocal && Type2IsLocal)
-    {
-        LocalType = Type2;
-        ExternalType = Type1;
-    }
-    else
-    {
-        return (Type1 == Type2);
-    }
-
-    switch (LocalType)
-    {
-        case ACPI_TYPE_LOCAL_REGION_FIELD:
-        case ACPI_TYPE_LOCAL_BANK_FIELD:
-        case ACPI_TYPE_LOCAL_INDEX_FIELD:
-
-            LocalType = ACPI_TYPE_FIELD_UNIT;
-            break;
-
-        default:
-            break;
-    }
-
-    return (LocalType == ExternalType);
-}
-
-
-/*******************************************************************************
- *
- * FUNCTION:    LdAnalyzeExternals
- *
- * PARAMETERS:  Node            - Node that represents the named object
- *              Op              - Named object declaring this named object
- *              ExternalOpType  - Type of ExternalOp
- *              WalkState       - Current WalkState
- *
- * RETURN:      Status
- *
- * DESCRIPTION: Node and Op represents an identically named object declaration
- *              that is either declared by the ASL external keyword or declared
- *              by operators that declare named objects (i.e. Name, Device,
- *              OperationRegion, and etc.). This function ensures that the
- *              declarations do not contradict each other.
- *
- ******************************************************************************/
-
-static ACPI_STATUS
-LdAnalyzeExternals (
-    ACPI_NAMESPACE_NODE     *Node,
-    ACPI_PARSE_OBJECT       *Op,
-    ACPI_OBJECT_TYPE        ExternalOpType,
-    ACPI_WALK_STATE         *WalkState)
-{
-    ACPI_STATUS             Status = AE_OK;
-    ACPI_OBJECT_TYPE        ActualExternalOpType;
-    ACPI_OBJECT_TYPE        ActualOpType;
-    ACPI_PARSE_OBJECT       *ExternalOp;
-    ACPI_PARSE_OBJECT       *ActualOp;
-
-
-    /*
-     * The declaration represented by Node and Op must have the same type.
-     * The type of the external Op is represented by ExternalOpType. However,
-     * the type of the pre-existing declaration depends on whether if Op
-     * is an external declaration or an actual declaration.
-     */
-    if (Op->Asl.ParseOpcode == PARSEOP_EXTERNAL)
-    {
-        ActualExternalOpType = ExternalOpType;
-        ActualOpType = Node->Type;
-    }
-    else
-    {
-        ActualExternalOpType = Node->Type;
-        ActualOpType = ExternalOpType;
-    }
-
-    if ((ActualOpType != ACPI_TYPE_ANY) &&
-        (ActualExternalOpType != ACPI_TYPE_ANY) &&
-        !LdMatchExternType (ActualExternalOpType, ActualOpType))
-    {
-        if (Op->Asl.ParseOpcode == PARSEOP_EXTERNAL &&
-            Node->Op->Asl.ParseOpcode == PARSEOP_EXTERNAL)
-        {
-            AslDualParseOpError (ASL_WARNING,
-                ASL_MSG_DUPLICATE_EXTERN_MISMATCH, Op, NULL,
-                ASL_MSG_DUPLICATE_EXTERN_FOUND_HERE, Node->Op, NULL);
-        }
-        else
-        {
-            if (Op->Asl.ParseOpcode == PARSEOP_EXTERNAL &&
-                Node->Op->Asl.ParseOpcode != PARSEOP_EXTERNAL)
-            {
-                ExternalOp = Op;
-                ActualOp = Node->Op;
-            }
-            else
-            {
-                ExternalOp = Node->Op;
-                ActualOp = Op;
-            }
-            AslDualParseOpError (ASL_WARNING,
-                ASL_MSG_DECLARATION_TYPE_MISMATCH, ExternalOp, NULL,
-                ASL_MSG_TYPE_MISMATCH_FOUND_HERE, ActualOp, NULL);
-        }
-    }
-
-    /* Set the object type of the external */
-
-    if ((Node->Flags & ANOBJ_IS_EXTERNAL) &&
-        (Op->Asl.ParseOpcode != PARSEOP_EXTERNAL))
-    {
-        /*
-         * Allow one create on an object or segment that was
-         * previously declared External
-         */
-        Node->Flags &= ~ANOBJ_IS_EXTERNAL;
-        Node->Type = (UINT8) ExternalOpType;
-
-        /* Just retyped a node, probably will need to open a scope */
-
-        if (AcpiNsOpensScope (ExternalOpType))
-        {
-            Status = AcpiDsScopeStackPush (Node, ExternalOpType, WalkState);
-            if (ACPI_FAILURE (Status))
-            {
-                return (Status);
-            }
-        }
-
-        Status = AE_OK;
-    }
-    else if (!(Node->Flags & ANOBJ_IS_EXTERNAL) &&
-             (Op->Asl.ParseOpcode == PARSEOP_EXTERNAL))
-    {
-        /*
-         * Allow externals in same scope as the definition of the
-         * actual object. Similar to C. Allows multiple definition
-         * blocks that refer to each other in the same file.
-         */
-        Status = AE_OK;
-    }
-    else if ((Node->Flags & ANOBJ_IS_EXTERNAL) &&
-             (Op->Asl.ParseOpcode == PARSEOP_EXTERNAL) &&
-             (ExternalOpType == ACPI_TYPE_ANY))
-    {
-        /* Allow update of externals of unknown type. */
-
-        Node->Type = (UINT8) ExternalOpType;
-        Status = AE_OK;
-    }
-
-    return (Status);
-}
-
-
-/*******************************************************************************
- *
- * FUNCTION:    LdCheckSpecialNames
- *
- * PARAMETERS:  Node        - Node that represents the named object
- *              Op          - Named object declaring this named object
- *
- * RETURN:      None
- *
- * DESCRIPTION: Check if certain named objects are declared in the incorrect
- *              scope. Special named objects are listed in
- *              AslGbl_SpecialNamedObjects and can only be declared at the root
- *              scope. _UID inside of a processor declaration must not be a
- *              string.
- *
- ******************************************************************************/
-
-static void
-LdCheckSpecialNames (
-    ACPI_NAMESPACE_NODE     *Node,
-    ACPI_PARSE_OBJECT       *Op)
-{
-    UINT32                  i;
-
-
-    for (i = 0; i < MAX_SPECIAL_NAMES; i++)
-    {
-        if (ACPI_COMPARE_NAMESEG(Node->Name.Ascii, AslGbl_SpecialNamedObjects[i]) &&
-            Node->Parent != AcpiGbl_RootNode)
-        {
-            AslError (ASL_ERROR, ASL_MSG_INVALID_SPECIAL_NAME, Op, Op->Asl.ExternalName);
-            return;
-        }
-    }
-
-    if (ACPI_COMPARE_NAMESEG (Node->Name.Ascii, "_UID") &&
-        Node->Parent->Type == ACPI_TYPE_PROCESSOR &&
-        Node->Type == ACPI_TYPE_STRING)
-    {
-        AslError (ASL_ERROR, ASL_MSG_INVALID_PROCESSOR_UID , Op, "found a string");
-    }
-}
-
-
 /*******************************************************************************
  *
  * FUNCTION:    LdNamespace2Begin
@@ -1342,10 +1110,10 @@ LdNamespace2Begin (
 
     if (Op->Asl.ParseOpcode == PARSEOP_ALIAS)
     {
-        /*
-         * Complete the alias node by getting and saving the target node.
-         * First child is the alias target
-         */
+        /* Complete the alias node by getting and saving the target node */
+
+        /* First child is the alias target */
+
         Arg = Op->Asl.Child;
 
         /* Get the target pathname */
@@ -1369,34 +1137,18 @@ LdNamespace2Begin (
         {
             if (Status == AE_NOT_FOUND)
             {
-                /* Standalone NameSeg vs. NamePath */
+                AslError (ASL_ERROR, ASL_MSG_NOT_FOUND, Op,
+                    Op->Asl.ExternalName);
 
-                if (strlen (Arg->Asl.ExternalName) == ACPI_NAMESEG_SIZE)
-                {
-                    AslError (ASL_ERROR, ASL_MSG_NOT_FOUND, Op,
-                        Arg->Asl.ExternalName);
-                }
-                else
-                {
-                    AslError (ASL_ERROR, ASL_MSG_NAMEPATH_NOT_EXIST, Op,
-                        Arg->Asl.ExternalName);
-                }
-
-#if 0
-/*
- * NOTE: Removed 10/2018 to enhance compiler error reporting. No
- * regressions seen.
- */
                 /*
                  * The name was not found, go ahead and create it.
                  * This prevents more errors later.
                  */
                 Status = AcpiNsLookup (WalkState->ScopeInfo, Path,
-                    ACPI_TYPE_ANY, ACPI_IMODE_LOAD_PASS1,
-                    ACPI_NS_NO_UPSEARCH, WalkState, &Node);
-#endif
-                return (Status);
-/* Removed: return (AE_OK)*/
+                    ACPI_TYPE_ANY,
+                    ACPI_IMODE_LOAD_PASS1, ACPI_NS_NO_UPSEARCH,
+                    WalkState, &(Node));
+                return (AE_OK);
             }
 
             AslCoreSubsystemError (Op, Status,
