@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2020, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2018, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -154,7 +154,7 @@
 
 /* Current ACPICA subsystem version in YYYYMMDD format */
 
-#define ACPI_CA_VERSION                 0x20200430
+#define ACPI_CA_VERSION                 0x20180508
 
 #include "acconfig.h"
 #include "actypes.h"
@@ -300,6 +300,21 @@ ACPI_INIT_GLOBAL (UINT8,            AcpiGbl_CopyDsdtLocally, FALSE);
  */
 ACPI_INIT_GLOBAL (UINT8,            AcpiGbl_DoNotUseXsdt, FALSE);
 
+/*
+ * Optionally support group module level code.
+ * NOTE, this is essentially obsolete and will be removed soon
+ * (01/2018).
+ */
+ACPI_INIT_GLOBAL (UINT8,            AcpiGbl_GroupModuleLevelCode, FALSE);
+
+/*
+ * Optionally support module level code by parsing an entire table as
+ * a method as it is loaded. Default is TRUE.
+ * NOTE, this is essentially obsolete and will be removed soon
+ * (01/2018).
+ */
+ACPI_INIT_GLOBAL (UINT8,            AcpiGbl_ExecuteTablesAsMethods, TRUE);
+
 /*
  * Optionally use 32-bit FADT addresses if and when there is a conflict
  * (address mismatch) between the 32-bit and 64-bit versions of the
@@ -446,9 +461,6 @@ ACPI_GLOBAL (BOOLEAN,               AcpiGbl_SystemAwakeAndRunning);
 #define ACPI_HW_DEPENDENT_RETURN_OK(Prototype) \
     ACPI_EXTERNAL_RETURN_OK(Prototype)
 
-#define ACPI_HW_DEPENDENT_RETURN_UINT32(prototype) \
-    ACPI_EXTERNAL_RETURN_UINT32(prototype)
-
 #define ACPI_HW_DEPENDENT_RETURN_VOID(Prototype) \
     ACPI_EXTERNAL_RETURN_VOID(Prototype)
 
@@ -459,9 +471,6 @@ ACPI_GLOBAL (BOOLEAN,               AcpiGbl_SystemAwakeAndRunning);
 #define ACPI_HW_DEPENDENT_RETURN_OK(Prototype) \
     static ACPI_INLINE Prototype {return(AE_OK);}
 
-#define ACPI_HW_DEPENDENT_RETURN_UINT32(prototype) \
-    static ACPI_INLINE prototype {return(0);}
-
 #define ACPI_HW_DEPENDENT_RETURN_VOID(Prototype) \
     static ACPI_INLINE Prototype {return;}
 
@@ -664,13 +673,7 @@ AcpiInstallTable (
 ACPI_EXTERNAL_RETURN_STATUS (
 ACPI_STATUS
 AcpiLoadTable (
-    ACPI_TABLE_HEADER       *Table,
-    UINT32                  *TableIdx))
-
-ACPI_EXTERNAL_RETURN_STATUS (
-ACPI_STATUS
-AcpiUnloadTable (
-    UINT32                  TableIndex))
+    ACPI_TABLE_HEADER       *Table))
 
 ACPI_EXTERNAL_RETURN_STATUS (
 ACPI_STATUS
@@ -1088,12 +1091,6 @@ AcpiGetGpeStatus (
     UINT32                  GpeNumber,
     ACPI_EVENT_STATUS       *EventStatus))
 
-ACPI_HW_DEPENDENT_RETURN_UINT32 (
-UINT32
-AcpiDispatchGpe (
-    ACPI_HANDLE             GpeDevice,
-    UINT32                  GpeNumber))
-
 ACPI_HW_DEPENDENT_RETURN_STATUS (
 ACPI_STATUS
 AcpiDisableAllGpes (
@@ -1109,10 +1106,6 @@ ACPI_STATUS
 AcpiEnableAllWakeupGpes (
     void))
 
-ACPI_HW_DEPENDENT_RETURN_UINT32 (
-    UINT32                  AcpiAnyGpeStatusSet (
-    void))
-
 ACPI_HW_DEPENDENT_RETURN_STATUS (
 ACPI_STATUS
 AcpiGetGpeDevice (
@@ -1351,16 +1344,6 @@ AcpiBiosError (
     const char              *Format,
     ...))
 
-ACPI_MSG_DEPENDENT_RETURN_VOID (
-ACPI_PRINTF_LIKE(4)
-void  ACPI_INTERNAL_VAR_XFACE
-AcpiBiosException (
-    const char              *ModuleName,
-    UINT32                  LineNumber,
-    ACPI_STATUS             Status,
-    const char              *Format,
-    ...))
-
 ACPI_MSG_DEPENDENT_RETURN_VOID (
 ACPI_PRINTF_LIKE(3)
 void ACPI_INTERNAL_VAR_XFACE
