@@ -8,7 +8,7 @@
  *
  * 1. Copyright Notice
  *
- * Some or all of this work - Copyright (c) 1999 - 2020, Intel Corp.
+ * Some or all of this work - Copyright (c) 1999 - 2018, Intel Corp.
  * All rights reserved.
  *
  * 2. License
@@ -167,7 +167,7 @@ typedef struct osl_table_info
 {
     struct osl_table_info   *Next;
     UINT32                  Instance;
-    char                    Signature[ACPI_NAMESEG_SIZE];
+    char                    Signature[ACPI_NAME_SIZE];
 
 } OSL_TABLE_INFO;
 
@@ -484,7 +484,7 @@ OslAddTableToList (
         return (AE_NO_MEMORY);
     }
 
-    ACPI_COPY_NAMESEG (NewInfo->Signature, Signature);
+    ACPI_MOVE_NAME (NewInfo->Signature, Signature);
 
     if (!Gbl_TableListHead)
     {
@@ -495,7 +495,7 @@ OslAddTableToList (
         Next = Gbl_TableListHead;
         while (1)
         {
-            if (ACPI_COMPARE_NAMESEG (Next->Signature, Signature))
+            if (ACPI_COMPARE_NAME (Next->Signature, Signature))
             {
                 if (Next->Instance == Instance)
                 {
@@ -1050,11 +1050,11 @@ OslGetBiosTable (
 
     /* Handle special tables whose addresses are not in RSDT/XSDT */
 
-    if (ACPI_COMPARE_NAMESEG (Signature, ACPI_RSDP_NAME) ||
-        ACPI_COMPARE_NAMESEG (Signature, ACPI_SIG_RSDT) ||
-        ACPI_COMPARE_NAMESEG (Signature, ACPI_SIG_XSDT) ||
-        ACPI_COMPARE_NAMESEG (Signature, ACPI_SIG_DSDT) ||
-        ACPI_COMPARE_NAMESEG (Signature, ACPI_SIG_FACS))
+    if (ACPI_COMPARE_NAME (Signature, ACPI_RSDP_NAME) ||
+        ACPI_COMPARE_NAME (Signature, ACPI_SIG_RSDT) ||
+        ACPI_COMPARE_NAME (Signature, ACPI_SIG_XSDT) ||
+        ACPI_COMPARE_NAME (Signature, ACPI_SIG_DSDT) ||
+        ACPI_COMPARE_NAME (Signature, ACPI_SIG_FACS))
     {
 
 FindNextInstance:
@@ -1066,7 +1066,7 @@ FindNextInstance:
          * careful about the FADT length and validate table addresses.
          * Note: The 64-bit addresses have priority.
          */
-        if (ACPI_COMPARE_NAMESEG (Signature, ACPI_SIG_DSDT))
+        if (ACPI_COMPARE_NAME (Signature, ACPI_SIG_DSDT))
         {
             if (CurrentInstance < 2)
             {
@@ -1082,7 +1082,7 @@ FindNextInstance:
                 }
             }
         }
-        else if (ACPI_COMPARE_NAMESEG (Signature, ACPI_SIG_FACS))
+        else if (ACPI_COMPARE_NAME (Signature, ACPI_SIG_FACS))
         {
             if (CurrentInstance < 2)
             {
@@ -1098,7 +1098,7 @@ FindNextInstance:
                 }
             }
         }
-        else if (ACPI_COMPARE_NAMESEG (Signature, ACPI_SIG_XSDT))
+        else if (ACPI_COMPARE_NAME (Signature, ACPI_SIG_XSDT))
         {
             if (!Gbl_Revision)
             {
@@ -1110,7 +1110,7 @@ FindNextInstance:
                     (ACPI_PHYSICAL_ADDRESS) Gbl_Rsdp.XsdtPhysicalAddress;
             }
         }
-        else if (ACPI_COMPARE_NAMESEG (Signature, ACPI_SIG_RSDT))
+        else if (ACPI_COMPARE_NAME (Signature, ACPI_SIG_RSDT))
         {
             if (CurrentInstance == 0)
             {
@@ -1206,7 +1206,7 @@ FindNextInstance:
 
             /* Does this table match the requested signature? */
 
-            if (!ACPI_COMPARE_NAMESEG (MappedTable->Signature, Signature))
+            if (!ACPI_COMPARE_NAME (MappedTable->Signature, Signature))
             {
                 OslUnmapTable (MappedTable);
                 MappedTable = NULL;
@@ -1277,7 +1277,7 @@ OslListCustomizedTables (
 {
     void                    *TableDir;
     UINT32                  Instance;
-    char                    TempName[ACPI_NAMESEG_SIZE];
+    char                    TempName[ACPI_NAME_SIZE];
     char                    *Filename;
     ACPI_STATUS             Status = AE_OK;
 
@@ -1377,7 +1377,7 @@ OslMapTable (
                 return (AE_BAD_SIGNATURE);
             }
         }
-        else if (!ACPI_COMPARE_NAMESEG (Signature, MappedTable->Signature))
+        else if (!ACPI_COMPARE_NAME (Signature, MappedTable->Signature))
         {
             AcpiOsUnmapMemory (MappedTable, sizeof (ACPI_TABLE_HEADER));
             return (AE_BAD_SIGNATURE);
@@ -1457,18 +1457,18 @@ OslTableNameFromFile (
 
     /* Ignore meaningless files */
 
-    if (strlen (Filename) < ACPI_NAMESEG_SIZE)
+    if (strlen (Filename) < ACPI_NAME_SIZE)
     {
         return (AE_BAD_SIGNATURE);
     }
 
     /* Extract instance number */
 
-    if (isdigit ((int) Filename[ACPI_NAMESEG_SIZE]))
+    if (isdigit ((int) Filename[ACPI_NAME_SIZE]))
     {
-        sscanf (&Filename[ACPI_NAMESEG_SIZE], "%u", Instance);
+        sscanf (&Filename[ACPI_NAME_SIZE], "%u", Instance);
     }
-    else if (strlen (Filename) != ACPI_NAMESEG_SIZE)
+    else if (strlen (Filename) != ACPI_NAME_SIZE)
     {
         return (AE_BAD_SIGNATURE);
     }
@@ -1479,7 +1479,7 @@ OslTableNameFromFile (
 
     /* Extract signature */
 
-    ACPI_COPY_NAMESEG (Signature, Filename);
+    ACPI_MOVE_NAME (Signature, Filename);
     return (AE_OK);
 }
 
@@ -1549,7 +1549,7 @@ OslReadTableFromFile (
                 goto Exit;
             }
         }
-        else if (!ACPI_COMPARE_NAMESEG (Signature, Header.Signature))
+        else if (!ACPI_COMPARE_NAME (Signature, Header.Signature))
         {
             fprintf (stderr, "Incorrect signature: Expecting %4.4s, found %4.4s\n",
                 Signature, Header.Signature);
@@ -1629,7 +1629,7 @@ OslGetCustomizedTable (
 {
     void                    *TableDir;
     UINT32                  CurrentInstance = 0;
-    char                    TempName[ACPI_NAMESEG_SIZE];
+    char                    TempName[ACPI_NAME_SIZE];
     char                    TableFilename[PATH_MAX];
     char                    *Filename;
     ACPI_STATUS             Status;
@@ -1649,7 +1649,7 @@ OslGetCustomizedTable (
     {
         /* Ignore meaningless files */
 
-        if (!ACPI_COMPARE_NAMESEG (Filename, Signature))
+        if (!ACPI_COMPARE_NAME (Filename, Signature))
         {
             continue;
         }
